;
; SRAM_Test1
;
; Purpose:
;    -> manually test the SRAM
;    -> 3 encoders and 2 buttons conntected to a DIN are required
;	the first two encoders select the address to read/write,
;       the third encoder selects the value to write
;	first button: write value at address
;	second button: read from address
;
; ==========================================================================
;
; Copyright (C) 2003  Thorsten Klose (Thorsten.Klose@gmx.de)
; Copyright (C) 2005  TL (tl@automatic-brain.de)
; 
; ==========================================================================
; 
; This file is part of an MIOS example application
;
; This application is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This application is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this application; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; ==========================================================================
;
; Required tools:
;    o Download the free available MPLAB IDE environment from the Microchip
;      Homepage http://www.microchip.com
;    o Download and install Active perl from http://www.perl.com
;    o Download and install MIDI-Ox from http://www.midiox.com
;    o Download and install the MIOS Bootstrap loader from http://www.ucapps.de and
;      
; Following steps are required to reassemble the source code:
;    o Start MPLAB IDE
;    o Open the project file (*.mcp) from this archive
;    o Open the main.asm file, here you can find all available include files 
;    o Change the files for your needs 
;    o Start "Project->Make" or just press the F10 button 
;    o After that, a new main.hex file can be found in the directory
;    o convert the main.hex file to a .syx file in the command shell with:
;      perl hex2syx.pl main.hex
;      or just click on "convert.bat"
;    o thereafter a main.syx file can be found in the directory. It
;      can be uploaded to the MIDIbox NG with MIDI-Ox (see additional
;      instructions in the MIOS bootloader archive)
;
; ==========================================================================
;; ---[ MIOS header file ]---
#include "mios.h"

;; ---[ useful macros ]---
#include "macros.h"

;; ---[ vectors to MIOS functions (never change!) ]---
#include "mios_vectors.inc"

;; ---[ user hooks (never change!) ]---
#include "user_vectors.inc"

;; ==========================================================================
;;  General Application Settings
;; ==========================================================================

;; ---[ variables used by application ]---
#include "app_defines.h"

; ==========================================================================

;; ---[ configuration table for MIDI processor and rotary encoders ]---
#include "mios_tables.inc"

;; ---[ Custom LCD driver ]---
#include "app_lcd.inc"

#include "sram.inc"

;; ==========================================================================
;;  All MIOS hooks in one file
;; ==========================================================================

;; --------------------------------------------------------------------------
;;  This function is called by MIOS after startup to initialize the 
;;  application
;; --------------------------------------------------------------------------
USER_Init
	;; define number of shift registers: for 128 IOs, we need
	;; 1 registers. Btw.: thats the maximum number of supported DIN/DOUTs
	movlw	1
	call	MIOS_SRIO_NumberSet

	;; define the update frequency (latency) of DIN/DOUTs in mS
	movlw	1 ; ms
	call	MIOS_SRIO_UpdateFrqSet

	call SRAM_Init
        movlw   0x00
        movwf   SRAM_ADDR
        movwf   SRAM_ADDR+1
        movwf   SRAM_ADDR+2

        ;; write at internal SRAM page memory in PIC a test pattern
        movlw   0xaa
        movwf   TMP1
        movlw   0x00
        lfsr    FSR0, SRAM_MAP
Init_Loop
	movff 	TMP1, POSTINC0
	cpfseq  FSR0L
	bra     Init_Loop
        
        ; write test pattern into first 256 bytes of external SRAM
        call    SRAM_Write_Page
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS in the mainloop when nothing else is to do
;; --------------------------------------------------------------------------
USER_Tick
	return


;; --------------------------------------------------------------------------
;;  This function is periodically called by MIOS. The frequency has to be
;;  initialized with MIOS_Timer_Set
;;  Note that this is an interrupt service routine! Use FSR2 instead of FSR0
;;  and IRQ_TMPx instead of TMPx -- and make the routine as fast as possible!
;; --------------------------------------------------------------------------
USER_Timer
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a debug command has been received
;;  via SysEx
;;  Input:
;;     o WREG, MIOS_PARAMETER1, MIOS_PARAMETER2, MIOS_PARAMETER3 like
;;       specified in the debug command
;;  Output:
;;     o return values WREG, MIOS_PARAMETER1, MIOS_PARAMETER2, MIOS_PARAMETER3
;; --------------------------------------------------------------------------
USER_MPROC_DebugTrigger
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when the display content should be 
;;  initialized. Thats the case during startup and after a temporary message
;;  has been printed on the screen
;; --------------------------------------------------------------------------
USER_DISPLAY_Init
	call	MIOS_LCD_Clear			; clear LCD

	TABLE_ADDR TEXT_WELCOME_0		; print welcome message
	call	MIOS_LCD_PrintString

	return

TEXT_WELCOME_0	STRING 14, 0x00, "MBHP SRAM Test"

;; --------------------------------------------------------------------------
;;  This function is called in the mainloop when no temporary message is shown
;;  on screen. Print the realtime messages here
;; --------------------------------------------------------------------------
USER_DISPLAY_Tick
	;; leave routine if update has not been requested
	IFCLR	DISPLAY_UPDATE_REQ, 0, return

	;; clear the request flag so that this routine will not be processed
	;; until the flag has been set again
	bcf	DISPLAY_UPDATE_REQ, 0

	call	MIOS_LCD_Clear			; clear LCD

	;; print the value of the last turned encoder:

	TABLE_ADDR TEXT_ADDR_0		
	call	MIOS_LCD_PrintString

	movf	SRAM_ADDR, W
	call	MIOS_LCD_PrintHex2
	movf	SRAM_ADDR+1, W
	call	MIOS_LCD_PrintHex2
	movf	SRAM_ADDR+2, W
	call	MIOS_LCD_PrintHex2

	TABLE_ADDR TEXT_DATA_0		
	call	MIOS_LCD_PrintString

	movf	SRAM_DATA_W, W	
	call	MIOS_LCD_PrintHex2

	return

TEXT_ADDR_0	STRING 9, 0x00, "address: "
TEXT_DATA_0	STRING 7, 0x40, "value: "

;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a complete MIDI event has been received
;;  Input:
;;     o first  MIDI event byte in MIOS_PARAMETER1
;;     o second MIDI event byte in MIOS_PARAMETER2
;;     o third  MIDI event byte in MIOS_PARAMETER3
;; --------------------------------------------------------------------------
USER_MPROC_NotifyReceivedEvent
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a MIDI event has been received
;;  which has been specified in the MIOS_MPROC_EVENT_TABLE table
;;  Input:
;;     o number of entry in WREG
;;     o first  MIDI event byte in MIOS_PARAMETER1
;;     o second MIDI event byte in MIOS_PARAMETER2
;;     o third  MIDI event byte in MIOS_PARAMETER3
;; --------------------------------------------------------------------------
USER_MPROC_NotifyFoundEvent
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a MIDI event has not been completly
;;  received within 2 seconds
;; --------------------------------------------------------------------------
USER_MPROC_NotifyTimeout
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a MIDI byte has been received
;;  Input:
;;     o received MIDI byte in WREG and MIOS_PARAMETER1
;; --------------------------------------------------------------------------
USER_MPROC_NotifyReceivedByte
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS when an button has been toggled
;;  Input:
;;     o Button number in WREG and MIOS_PARAMETER1
;;     o Button value MIOS_PARAMETER2:
;;       - 1 if button has been released (=5V)
;;       - 0 if button has been pressed (=0V)
;; --------------------------------------------------------------------------
USER_DIN_NotifyToggle
	IFCLR MIOS_PARAMETER2, 0, goto pressed
	return
pressed
        ; write button
	movlw	0x04
	cpfseq	MIOS_PARAMETER1
	goto not_button5

	call	MIOS_LCD_Clear			; clear LCD

	TABLE_ADDR TEXT_WRITE_0		; print write message
	call	MIOS_LCD_PrintString

	movf	SRAM_ADDR, W
	call	MIOS_LCD_PrintHex2
	movf	SRAM_ADDR+1, W
	call	MIOS_LCD_PrintHex2
	movf	SRAM_ADDR+2, W
	call	MIOS_LCD_PrintHex2

	movlw	'-'
	call	MIOS_LCD_PrintChar
	movlw	'>'
	call	MIOS_LCD_PrintChar

	movf	SRAM_DATA_W, W
	call	MIOS_LCD_PrintHex2

        movff   SRAM_DATA_W, MIOS_PARAMETER1
	call SRAM_Write

        return

not_button5
        ; read button
	movlw	0x05
	cpfseq	MIOS_PARAMETER1
	goto din_done

	call	MIOS_LCD_Clear			; clear LCD

	TABLE_ADDR TEXT_READ_0		; print read message
	call	MIOS_LCD_PrintString

	movf	SRAM_ADDR, W
	call	MIOS_LCD_PrintHex2
	movf	SRAM_ADDR+1, W
	call	MIOS_LCD_PrintHex2
	movf	SRAM_ADDR+2, W
	call	MIOS_LCD_PrintHex2

	movlw	'-'
	call	MIOS_LCD_PrintChar
	movlw	'>'
	call	MIOS_LCD_PrintChar

	call	SRAM_Read
        movff   MIOS_PARAMETER1, SRAM_DATA_R

	movf	SRAM_DATA_R, W
	call	MIOS_LCD_PrintHex2
	; read page from ext. SRAM
        call    SRAM_Read_Page

din_done
	return

TEXT_WRITE_0	STRING 7, 0x00, "write: "
TEXT_READ_0	STRING 7, 0x00, " read: "

;; --------------------------------------------------------------------------
;;  This function is called by MIOS when an encoder has been moved
;;  Input:
;;     o Encoder number in WREG and MIOS_PARAMETER1
;;     o signed incrementer value in MIOS_PARAMETER2:
;;       - is positive when encoder has been turned clockwise
;;       - is negative when encoder has been turned counter clockwise
;; --------------------------------------------------------------------------
USER_ENC_NotifyChange
	;; an encoder has been moved. 
	;; Send a MIDI event (here: B0 07 <value> == CC #7 == volume channel 0)

	;; save encoder number in ENC_NUMBER
	movff	MIOS_PARAMETER1, ENC_NUMBER
	;; save incrementer in ENC_INC
	movff	MIOS_PARAMETER2, ENC_INC

	;; now we have to add (or subtract) the incrementer from the absolute value,
	;; which has been stored in ENC_VALUE_16BIT_L

	;; the simplest solution is the use of MIOS_HLP_16bitAddSaturate, which
	;; can handle with values up to 16 bit!!! :-)
	;; please refer to the function documentation for the input/output parameters!

	;; set pointer to ENC_VALUE_16BIT_L
	;; (it is located to an even address, thats important when this function is used!)
	;; (and since we are modifying a 16-bit value, the next address to the low-byte
	;; is used as high-byte -> see app_defines.h)
	movlw	0x00
	cpfseq	ENC_NUMBER
	goto	not_enc1

	lfsr	FSR1, ENC_VALUE_16BIT_L
	;; set max value
	movlw	0xff	; low-byte of max value
	movwf	MIOS_PARAMETER1
	movlw	0x00	; high-byte of max value
	movwf	MIOS_PARAMETER2
	;; get incrementer (which has been stored in ENC_INC)
	movf	ENC_INC, W
	;; call routine
	call	MIOS_HLP_16bitAddSaturate
	;; now [FSR1] = INDF1 = ENC_VALUE_16BIT_L contains the result
	;; MIOS_PARAMETER1[0] is set when value has been changed

	;; exit routine if value has not been changed
	IFCLR	MIOS_PARAMETER1, 0, return

	movff	ENC_VALUE_16BIT_L, SRAM_ADDR+2

	;; request a display update by setting DISPLAY_UPDATE_REQ,0
	;; USER_DISPLAY_Tick polls this flag!
	bsf	DISPLAY_UPDATE_REQ, 0

	return

not_enc1
	movlw	0x01
	cpfseq	ENC_NUMBER
	goto	not_enc2
	lfsr	FSR1, ENC_VALUE_16BIT_L
	;; set max value
	movlw	0xff	; low-byte of max value
	movwf	MIOS_PARAMETER1
	movlw	0x07	; high-byte of max value
	movwf	MIOS_PARAMETER2
	;; get incrementer (which has been stored in ENC_INC)
	movf	ENC_INC, W
	;; call routine
	call	MIOS_HLP_16bitAddSaturate
	;; now [FSR1] = INDF1 = ENC_VALUE_16BIT_L contains the result
	;; MIOS_PARAMETER1[0] is set when value has been changed

	;; exit routine if value has not been changed
	IFCLR	MIOS_PARAMETER1, 0, return

	movff	ENC_VALUE_16BIT_L, SRAM_ADDR+1
	movff	ENC_VALUE_16BIT_H, SRAM_ADDR


	;; request a display update by setting DISPLAY_UPDATE_REQ,0
	;; USER_DISPLAY_Tick polls this flag!
	bsf	DISPLAY_UPDATE_REQ, 0

	;; thats all
	return	

not_enc2
	movlw	0x02
	cpfseq	ENC_NUMBER
	goto	nothing
	lfsr	FSR1, ENC_VALUE_16BIT_L
	;; set max value
	movlw	0xff	; low-byte of max value
	movwf	MIOS_PARAMETER1
	movlw	0x00	; high-byte of max value
	movwf	MIOS_PARAMETER2
	;; get incrementer (which has been stored in ENC_INC)
	movf	ENC_INC, W
	;; call routine
	call	MIOS_HLP_16bitAddSaturate
	;; now [FSR1] = INDF1 = ENC_VALUE_16BIT_L contains the result
	;; MIOS_PARAMETER1[0] is set when value has been changed

	;; exit routine if value has not been changed
	IFCLR	MIOS_PARAMETER1, 0, return

	movff	ENC_VALUE_16BIT_L, SRAM_DATA_W

	;; request a display update by setting DISPLAY_UPDATE_REQ,0
	;; USER_DISPLAY_Tick polls this flag!
	bsf	DISPLAY_UPDATE_REQ, 0

	;; thats all

nothing
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS before the shift register are loaded
;;  Note that this is an interrupt service routine! Use FSR2 instead of FSR0
;;  and IRQ_TMPx instead of TMPx -- and make the routine as fast as possible
;; --------------------------------------------------------------------------
USER_SR_Service_Prepare
	return


;; --------------------------------------------------------------------------
;;  This function is called by MIOS after the shift register have been loaded
;;  Note that this is an interrupt service routine! Use FSR2 instead of FSR0
;;  and IRQ_TMPx instead of TMPx -- and make the routine as fast as possible
;; --------------------------------------------------------------------------
USER_SR_Service_Finish
	return

;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a pot has been moved
;;  Input:
;;     o Pot number in WREG and MIOS_PARAMETER1
;;     o LSB value in MIOS_PARAMETER2
;;     o MSB value in MIOS_PARAMETER3
;; --------------------------------------------------------------------------
USER_AIN_NotifyChange
	return

	END
