;
; MIOS Example Driver for SRAM Module
;
; 128k x 8 and 512k x 8 SRAM type are supported
; 
; the page read/write routines DON'T check the address range 
; for speed reasons
;
; You may need to modify the Variables section,
; depending on your application
;
; ==========================================================================
;
; Copyright (C) 2005 by TL (tl@automatic-brain.de)
;
; ==========================================================================
;
; This file is part of SRAM_Test1
;
; SRAM_Test1 is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; <application> is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with <application>; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; ==========================================================================


;; --------------------------------------------------------------------------
;;  User-Variables
;; --------------------------------------------------------------------------

;  WARNING, in appdefines.h these addresses must remain free
;    and allocated in the first 128 bytes of bank 0
SRAM_ADDR		EQU	0x07d	; A16 - A18
; SRAM_ADDR+1				; A8  - A15
; SRAM_ADDR+2				; A0  - A7

; the address in PIC data memory where to map a page (256bytes) from SRAM
; must be 256bytes aligned i.e. 0x100 0x200 0x300 0x400
#define SRAM_MAP 0x100

;; --------------------------------------------------------------------------
;;  Constants for CS/LE pins of SRAM and latches
;; --------------------------------------------------------------------------
#define SRAM_LATCH1_LE 0
#define SRAM_LATCH2_LE 1
#define SRAM_CS 2
#define SRAM_WE 3
#define SRAM_OE 5

;; --------------------------------------------------------------------------
;;  This function is called after startup to initialize the SRAM
;; --------------------------------------------------------------------------
SRAM_Init
	;; disable the ADC which allocates the analog pins
	movlw	0x07
	movwf	ADCON1

	; put PortA in output mode
	bcf		TRISA, 0
	bcf		TRISA, 1
	bcf		TRISA, 2
	bcf		TRISA, 3
	bcf		TRISA, 5
	; set control lines
	bsf		PORTA, SRAM_OE
	bsf		PORTA, SRAM_WE
	bsf		PORTA, SRAM_CS
	bcf		PORTA, SRAM_LATCH1_LE
	bcf		PORTA, SRAM_LATCH2_LE

	; put PortE in output mode
	bcf		TRISE, 0
	bcf		TRISE, 1
	bcf		TRISE, 2	

	
	return

;; --------------------------------------------------------------------------
;;  This function is called to read a byte from the SRAM
;;  Input:
;;    o address in SRAM_ADDR, SRAM_ADD+1 and SRAM_ADDR+2
;;      SRAM_ADDR   = A16-A18
;;      SRAM_ADDR+1 = A8-A15
;;      SRAM_ADDR+2 = A0-A7
;;  Output:
;;    o byte read from SRAM in MIOS_PARAMETER1
;; --------------------------------------------------------------------------
SRAM_Read
	; first latch
	movff 	SRAM_ADDR+2, PORTB
	bsf 	LATA, SRAM_LATCH1_LE
	bcf 	LATA, SRAM_LATCH1_LE

	; second latch
	movff 	SRAM_ADDR+1,PORTB
	bsf 	LATA, SRAM_LATCH2_LE
	bcf 	LATA, SRAM_LATCH2_LE

	; remaining address lines
	movff	SRAM_ADDR, PORTE

	; enable port read mode
	setf	TRISB

	IRQ_DISABLE

	bcf 	LATA, SRAM_CS
	bcf 	LATA, SRAM_OE

	nop

	; read data
	movff 	PORTB, MIOS_PARAMETER1

	bsf 	LATA, SRAM_OE
	bsf 	LATA, SRAM_CS

	IRQ_ENABLE

	; disable port read mode
	clrf	TRISB
	return


;; --------------------------------------------------------------------------
;;  This function is called to write a byte to the SRAM
;;  Input:
;;    o address in SRAM_ADDR, SRAM_ADD+1 and SRAM_ADDR+2
;;      SRAM_ADDR   = A18-A16
;;      SRAM_ADDR+1 = A8-A15 
;;      SRAM_ADDR+2 = A0-A7
;;    o byte to write in MIOS_PARAMETER1
;; --------------------------------------------------------------------------
SRAM_Write

	; first latch
	movff 	SRAM_ADDR+2, PORTB
	bsf 	LATA, SRAM_LATCH1_LE
	bcf 	LATA, SRAM_LATCH1_LE

	; second latch
	movff	SRAM_ADDR+1, PORTB
	bsf 	LATA, SRAM_LATCH2_LE
	bcf 	LATA, SRAM_LATCH2_LE

	; remaining address lines
	movff 	SRAM_ADDR, PORTE

	IRQ_DISABLE

	; write byte
	movff 	MIOS_PARAMETER1, PORTB

	bcf 	LATA, SRAM_CS
	bcf 	LATA, SRAM_WE

	nop

	bsf 	LATA, SRAM_WE
	bsf 	LATA, SRAM_CS

	IRQ_ENABLE

	return


;; --------------------------------------------------------------------------
;;  This function is called to read a page (256bytes) from the SRAM
;;  Input:
;;    o address in SRAM_ADDR, SRAM_ADD+1 and SRAM_ADDR+2
;;      SRAM_ADDR       = A16-A18
;;      SRAM_ADDR+1     = A8-A15
;;      SRAM_ADDR+2     = A0-A7
;;
;;     Notes:
;;      NO RANGE CHECKING is performed
;;      SRAM_ADDR has to be 256bytes aligned (SRAM_ADDR+2 is ignored)
;;  Output:
;;    o in mapped pic data memory the 256 bytes read from SRAM
;;
;;  Uses:
;;    o FSR0
;; --------------------------------------------------------------------------
SRAM_Read_Page
	; second latch
	movff 	SRAM_ADDR+1,PORTB
	bsf 	LATA, SRAM_LATCH2_LE
	bcf 	LATA, SRAM_LATCH2_LE

	; remaining address lines
	movff	SRAM_ADDR, PORTE

        movlw   0x00
        lfsr    FSR0, SRAM_MAP

SRAM_Read_Page_Loop
        ; first latch
        movff 	FSR0L, PORTB
	bsf 	LATA, SRAM_LATCH1_LE
	bcf 	LATA, SRAM_LATCH1_LE

	; enable port read mode
	setf	TRISB
	IRQ_DISABLE
	bcf 	LATA, SRAM_CS
	bcf 	LATA, SRAM_OE
	nop
	; read data into PIC memory
	movff 	PORTB, POSTINC0
	bsf 	LATA, SRAM_OE
	bsf 	LATA, SRAM_CS
	IRQ_ENABLE
	; disable port read mode
	clrf	TRISB
	cpfseq  FSR0L
	bra     SRAM_Read_Page_Loop
	return

;; --------------------------------------------------------------------------
;;  This function is called to write a page (256bytes) to the SRAM from
;;  PIC internal data memory
;;  Input:
;;    o address in SRAM_ADDR, SRAM_ADD+1 and SRAM_ADDR+2
;;      SRAM_ADDR       = A16-A18
;;      SRAM_ADDR+1     = A8-A15
;;      SRAM_ADDR+2     = A0-A7
;;
;;     Notes:
;;      NO RANGE CHECKING is performed
;;      SRAM_ADDR has to be 256bytes aligned (SRAM_ADDR+2 is ignored)
;;
;;  Uses:
;;    o FSR0
;; --------------------------------------------------------------------------
SRAM_Write_Page
	; second latch
	movff 	SRAM_ADDR+1,PORTB
	bsf 	LATA, SRAM_LATCH2_LE
	bcf 	LATA, SRAM_LATCH2_LE

	; remaining address lines
	movff	SRAM_ADDR, PORTE

        movlw   0x00
        lfsr    FSR0, SRAM_MAP

SRAM_Write_Page_Loop
        ; first latch
        movff 	FSR0L, PORTB
	bsf 	LATA, SRAM_LATCH1_LE
	bcf 	LATA, SRAM_LATCH1_LE
	IRQ_DISABLE
	; write byte
	movff 	POSTINC0, PORTB
	bcf 	LATA, SRAM_CS
	bcf 	LATA, SRAM_WE
	nop
	bsf 	LATA, SRAM_WE
	bsf 	LATA, SRAM_CS
	IRQ_ENABLE
	cpfseq  FSR0L
	bra     SRAM_Write_Page_Loop
	return


